package tk.mtantyukityuki.threeKingdoms.unit;

import tk.mtantyukityuki.threeKingdoms.model.*;
import tk.mtantyukityuki.threeKingdoms.unit.controller.*;
import tk.mtantyukityuki.threeKingdoms.unit.graphicsEngine.*;

import java.awt.*;

public class Ground extends Unit implements CreatureStorable {

	Structure	structure;	//̓yn̏ɗĂꂽzB(̏Ȃ狴nBnȂ瓹HEcEXEEĐՁEss)
	Creature	creature;	//̓yn̏ɒĂ鐶B(sƂRƂƂRnƂ)

	int	groundKind;

	static final int DEEP_WATER = -4;	//[
	static final int SHALLOW_WATER = -3;	//
	static final int SWAMP = -2;		//n
	static final int LOW_LAND = -1;		//n
	static final int PLAIN_LAND = 0;	//n
	static final int HIGH_LAND = 1;		//n
	static final int HILL_LAND = 2;		//u˒n
	static final int LOW_MOUNTAIN = 3;	//Rn
	static final int HIGH_MOUNTAIN = 4;	//R


	public Ground(int x, int y, Board board, int groundKind) {
		super(x,y, board, null, 0);

		this.groundKind = groundKind;
		if ( groundKind == SHALLOW_WATER ) {
			name = "󂢐n";
		}
		else {
			name = "n";
		}
	}

	public Ground(int x, int y, Board board, Direction direction, int groundKind) {
		super(x,y, board, direction, 0);

		this.groundKind = groundKind;
		if ( groundKind == SHALLOW_WATER ) {
			name = "󂢐n";
		}
		else { name = "n";
		}
	}

	public Structure getStructure() { return structure; }
	public void setStructure(Structure structure) { this.structure = structure; }
	public void removeStructure() { structure = null; }

	public Creature getCreature() { return creature; }

	public boolean contains(Creature creature) {
		if ( this.creature == creature ) {
				return true;
		} else {
			if ( structure instanceof CreatureStorable ) {
				return ( ((CreatureStorable)structure).contains( creature ) );
			} else {
				return false;
			}
		}
	}

	public boolean setCreature(Creature creature) {
		if ( structure instanceof CreatureStorable ) {
			return ((CreatureStorable)structure).setCreature( creature );
		} else {
			if ( this.creature == null ) {
				this.creature = creature;
				creature.setX(x);
				creature.setY(y);
				return true;
			} else {
				return false;
			}
		}
	}

	public boolean removeCreature() {
		if ( this.creature != null ) {
			this.creature = null;
			return true;
		} else {
			if ( structure instanceof CreatureStorable ) {
				return ((CreatureStorable)structure).removeCreature();
			} else {
				return false;
			}
		}
	}

	public boolean removeCreature(Creature creature) {
		if ( this.creature == creature ) {
			this.creature =  null;
			return true;
		} else {
			if ( structure instanceof CreatureStorable ) {
				return ((CreatureStorable)structure).removeCreature(creature);
			} else {
				return false;
			}
		}
	}

	public void fireCreatureAdded(Creature creature) {}
	public void fireCreatureRemoved(Creature creature) {}

	public int getGroundKind() { return groundKind; }

	/** ̒n`ƁA̒n`̏ɂzʉ߂̂ɂ́Aǂ̂炢̈ړ͂KvƂ邩B */
	public int getRestraint(Direction direction) {

		int length;

		/*
		 * XNEFAӂɕsɐi񂾏ꍇƁAΊpɕsɐi񂾏ꍇ̋̕␳B
		 * ӂɕsɐi񂾏ꍇ̋10ƂAΊpɕsɐi񂾏ꍇ̋14ƂB
		 * (1:1.41421356)
		 */
		switch ( direction.getDirection() ) {
			case Direction.UP_RIGHT :
			case Direction.UP_LEFT :
			case Direction.DOWN_RIGHT :
			case Direction.DOWN_LEFT :
				length = 14;
			default :
				length = 10;
		}

		if ( structure != null ) {
			return ( structure.getRestraint(direction) * length );
		} else {
			return ( getGroundRestraint(direction) * length );
		}
	}

	/** ̒n`ʉ߂̂ɂ́Aǂ̂炢̈ړ͂KvƂ邩B */
	public int getGroundRestraint(Direction direction) {

		int retRestraint;

		switch ( groundKind ) {
			case PLAIN_LAND:
				retRestraint = 2;
				break;
			case HILL_LAND:
			case LOW_LAND:
				retRestraint = 3;
				break;
			case HIGH_LAND:
			case SWAMP:
				retRestraint = 4;
				break;
			case LOW_MOUNTAIN:
			case SHALLOW_WATER:
				retRestraint = 5;
				break;
			case DEEP_WATER:
				retRestraint = 6;
				break;
			default:
				retRestraint = 4;
				break;
		}

		return retRestraint;
	}

	public int getOffensivePower(Direction attackedDirection) {
		if ( structure != null ) {
			return ( groundKind + structure.getOffensivePower(attackedDirection) );
		} else {
			return groundKind;
		}
	}
	public int getDefensivePower(Direction attackedDirection) {
		if ( structure != null ) {
			return ( groundKind + structure.getDefensivePower(attackedDirection) );
		} else {
			return groundKind;
		}
	}


	public void show(Graphics g, int x, int y, UnitGraphicsEngine uge, Component view, Direction direction) {
		uge.show(g, x,y, this, view, direction);
	}
	/** ̃jbgNbNꂽƂɂǂ̂悤ȓ邩̃_ufBXpb`ȃ\bhB*/
	public void clicked(UnitController controller) {
		controller.clicked(this);
	}
}
